/* Command line SF16-FMD2 radio control program
 *
 * (C) 1999 by Andy Ramensky, andy@clhs.kiev.ua
 *
 * 	www.clhs.kiev.ua/~andy
 */

#include <asm/io.h>		/* outb, outb_p                   */
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>

#define ENCODE(x) (int)((float)(x)*0.07985714+871.28571)
#define ENCODE1(x) (int)(((float)(x)-871.28571)/0.07985714)
#define myport 0x384

int inbits(int);
int ioperm(int, int, int);
void outbits(unsigned int, unsigned int);
void mute(void);
void unmute(void);
void searchup(void);
void searchdown(void);
void setfreq(int);
void readfreq(void);
void radioVol(int);
void printstatus(void);


void main(int argc, char *argv[])
{
    int freq1;
    float fr;
    if (ioperm(myport, 2, 1) < 0)
    {
	printf("IO operations not permited, become r00t :)!!!\n");
	exit(0);
    }
    printf("FMD2 ! On port - %x\n", myport);
    if (argc == 1)
	unmute();
    else
	if ((!strcmp(argv[1], "help")) || (!strcmp(argv[1], "-?")) || (!strcmp(argv[1], "-help")) || (!strcmp(argv[1], "--help")) || (!strcmp(argv[1], "-h")))
	{
	    printf("Usage:\n %s {-,+,D|d,U|u}\n %s [freq [volume]]\n", argv[0],argv[0]);
	    exit(0);
        }
        else 
	{
	    switch (*argv[1])
	    {
	    case '-':
		    mute();
		    break;
	    case '+':
		    unmute();
		    break;
	    case 'D':
	    case 'd':
		    searchdown();
		    readfreq();
		    break;
	    case 'U':
	    case 'u':
		    searchup();
		    readfreq();
		    break;
	    default:
		    sscanf(argv[1], "%f", &fr);
		    freq1 = fr * 1000;
		    if (freq1 < 84000)
			    freq1 = 84000;
		    else if (freq1 > 109000)
			    freq1 = 109000;
		    setfreq(freq1);
		    printf("Set freq %x %i \n", ENCODE(freq1), freq1);
		    if (argc == 3)
			if (isdigit(*argv[2]))
			    if (atoi(argv[2]) >= 0 && atoi(argv[2]) <= 255)
				radioVol(atoi(argv[2]));
			    else
				printf("%s: volume must be between 0 and 255...\n",argv[0]);
	}
    }
    ioperm(myport, 2, 0);
}

int inbits(int cnt)
{
    int readbyte = 0;
    while (cnt--)
	readbyte = inb(myport);
    return (readbyte);
}

void outbits(unsigned int bits, unsigned int data)
{
    int cntbits = bits;
    unsigned int tmpdata = data;
    while (bits--)
    {
	data = tmpdata;;
	data >>= bits;
	if (cntbits > 10)
	{
	    outb((data & 1), myport);
	    inbits(2);
	    outb((data & 1) | 2, myport);
	    inbits(2);
	    outb((data & 1), myport);
	    inbits(2);
	}
	else
	{
	    outb((data & 1), myport);
	    inbits(3);
	    outb((data & 1) | 2, myport);
	    inbits(3);
	    outb((data & 1), myport);
	    inbits(3);
	}
    }
}

void mute(void)
{
    outb(0x00, myport);
    inbits(4);
}

void unmute(void)
{
    outb(0x04, myport);
    inbits(4);
}

void searchdown(void)
{
    mute();
    outbits(0x9, 0x102);
    outbits(0x10, 0);
    usleep(400000);
    unmute();
}

void searchup(void)
{
    mute();
    outbits(0x9, 0x182);
    outbits(0x10, 0);
    usleep(400000);
    unmute();
}

void setfreq(int freq)
{
    mute();
    outbits(0x9, 2);
    outbits(0x10, ENCODE(freq));
    unmute();
}

void readfreq(void)
{
    int i, indata, freq1;
    do
    {
	outb(5, myport);
	inbits(4);
    }
    while (inbits(1)==0xf7);	//Search in progress ?
    usleep(400000);
    indata = 0;
    for (i = 0; i < 0x18; i++)
    {
	outb(0x7, myport);
	inbits(4);
	outb(0x5, myport);
	inbits(4);
	if (i > 8)
	{
	    indata <<= 1;
	    indata = indata + (inbits(1) & 1);
	}
    }
    freq1=ENCODE1(indata);
    if (freq1 < 84000)
    {
        freq1 = 84000;
        setfreq(freq1);
    }
    else if (freq1 > 109000)
	 {
	    freq1 = 109000;
            setfreq(freq1);
	 }
    printf("Set freq %i\n", freq1);
}

void radioVol(int vol)
{
    ioperm(0x224, 2, 1);
    outb(82, 0x224);
    outb(vol, 0x225);		// One channel
    outb(83, 0x224);
    outb(vol, 0x225);		// the second...
    ioperm(0x224, 2, 0);
}

void printstatus(void)
{
/*
 * I don`t know how to take status of radio card.
 */
}
